
// Demo and exercising of reverse_emulation

#include "reverse_emulation.h"

#include <stdio.h>
#include <assert.h>

enum { buf_size = 10 }; // simplest to think about

// Code that manages emulator_frame and buffer isn't necessary in an emulator. It
// allows this demo to show the timestamp of the frame being displayed.
typedef long timestamp_t;
static timestamp_t emulator_frame = 0;
static timestamp_t buffer [buf_size]; // allows us to print which frame is being displayed

void emu_save_state( int i )
{
	// Save state to slot [i]
	// ...
	
	printf( "Save state to slot %d\n", i );
}

void emu_restore_state( int i )
{
	// Restore state from slot [i]
	// ...
	
	emulator_frame = i * buf_size;
	printf( "Restore state from slot %d\n", i );
}

void emu_run_frame( int n )
{
	// Emulate and generate audio/video to buffer [n]
	// ...
	
	buffer [n] = emulator_frame++;
}

void emu_run_frame_invisibly()
{
	// Emulate without audio/video output
	// ...
	
	emulator_frame++;
}

void print_buffer()
{
	int i;
	for ( i = 0; i < buf_size; i++ )
	{
		int frame = buffer [i];
		printf( "Buffer [%d] holds frame %d.%d\n",
				i, frame / buf_size, frame % buf_size );
	}
}

void display_frame( int i )
{
	// Display saved frame 'i'
	// ...
	
	printf( "Display frame %d.%d (index %d)\n",
			buffer [i] / buf_size,
			buffer [i] % buf_size, i );
}

// Demo code

// Display next 'count' frames
void display_next( int count )
{
	while ( count-- )
		display_frame( re_next_frame() );
}

// Display previous 'count' frames
void display_prev( int count )
{
	while ( count-- )
		display_frame( re_prev_frame() );
}

int main()
{
	// Must be called when resetting emulator
	re_init( buf_size );
	
	// run for a bit
	display_next( 25 );
	
	// backing up disturbs buffer
	display_prev( 2 );
	
	// won't entirely fill buffer in an orderly way, so it will need to be refilled
	// when reversing next
	display_next( 9 );
	
	printf( "Should refill buffer\n" ); // shows up as several state restores
	display_prev( 2 );
	
	// this will entirely fill buffer, so it won't need to be refilled when reversing
	display_next( 10 );
	
	printf( "Shouldn't refill buffer\n" );
	display_prev( 2 );
	
	return 0;
}

